/*
 * RF430_Comm.c
 *
 * RF430CL331H Host Example Project
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/


#include "msp430.h"
#include "RF430_example.h"
#include "RF430_Comm.h"
#include <stdint.h>

#define START       	1
#define CONTINUE    	0
#define FAIL			0
#define PASS 			1
#define SET				1
#define CLEAR			0
#define I2C_NACK_RCVD	2
#define I2C_TRANSMIT	3
#define LPM_MODE		LPM0_bits
uint8_t  *PTxData;                     // Pointer to TX data
uint8_t *PRxData;                     // Pointer to RX data
uint16_t TXByteCtr;
uint16_t RXByteCtr;

uint8_t RF430_I2C_State;
uint8_t RF430_I2C_Start = 0;

//#define I2C_DELAY 				600
//#define PULSE_OUT  P1SEL &= ~BIT4; P1DIR |= BIT4; P1OUT|= BIT4  //scope interrupt
#define I2C_DELAY 			0
#define I2C_ALLOW				(I2C_READY_IN & I2C_READY_PIN)

/**************************************************************************************************************************************************
*   Read_Register
***************************************************************************************************************************************************
*
* Brief : Reads the register at reg_addr, returns the result
*
* Param[in] :   reg_addr: the address to read out data from
*
* Param[out]:   None
*
* Return :      uint16_t: the result from the register
**************************************************************************************************************************************************/
uint16_t Read_Register(uint16_t reg_addr)
{
    uint8_t TxAddr[2] = {0,0};
    uint8_t RxBuffer[2] = {0,0};

	TxAddr[0] = reg_addr >> 8; 		// MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	// LSB of address

    while(!I2C_ALLOW);		//wait until it is safe to transmit

    if(RF430_I2C_Write_Restart_Read((uint8_t *)TxAddr, 2, (uint8_t *)RxBuffer, 2) != FAIL)
        return RxBuffer[1] << 8 | RxBuffer[0];
    else
        __no_operation();

    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent

    return RxBuffer[1] << 8 | RxBuffer[0];
}

/**************************************************************************************************************************************************
*   Read_Continuous
***************************************************************************************************************************************************
*
* Brief : Continuous read data_length bytes and store in the area "read_data"
*
* Param[in] :   reg_addr: the address to read out data from
* 				data_length: the amount of data to read out
*
* Param[out]:   read_data: the array to store the read out data
*
* Return :      None
**************************************************************************************************************************************************/
void Read_Continuous(uint16_t reg_addr, uint8_t* read_data, uint16_t data_length)
{
    uint8_t TxAddr[2] = {0,0};

	TxAddr[0] = reg_addr >> 8; 		// MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	// LSB of address

    while(!I2C_ALLOW);		//wait until it is safe to transmit

    RF430_I2C_Write_Restart_Read((uint8_t  *)TxAddr, 2, (uint8_t *)read_data, data_length);

    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent

}

/**************************************************************************************************************************************************
*   Write_Register
***************************************************************************************************************************************************
*
* Brief : writes the register at reg_addr with a value
*
* Param[in] :   reg_addr: the address to write the data to
* 				value: the value to write to reg_addr
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void Write_Register(uint16_t reg_addr, uint16_t value)
{
    uint8_t TxData[4] = {0,0,0,0};

	TxData[0] = reg_addr >> 8; 		// MSB of address
	TxData[1] = reg_addr & 0xFF; 	// LSB of address
	TxData[2] = value & 0xFF;
	TxData[3] = value >> 8;

    while(!I2C_ALLOW);		//wait until it is safe to transmit

    RF430_I2C_Write((uint8_t  *)TxData, 4, START);

    RF430_I2C_Send_Stop();

    {
    	uint16_t delay = 10;
		while ((UCB0CTL1 & UCTXSTP) && delay)             // Ensure stop condition got sent, added timeout due to infinite loop when it should not be  AK 10-27-2013
		{
			__delay_cycles(10);
			delay--;
		}
    }
}

/**************************************************************************************************************************************************
*   Write_Continuous
***************************************************************************************************************************************************
*
* Brief : writes the register at reg_addr and incrementing addresses with the data at "write_data" of length data_length
*
* Param[in] :   reg_addr: the starting address to write the data
* 				write_data: an array of the data to write
* 				data_length: the amount of data to write
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void Write_Continuous(uint16_t reg_addr, uint8_t  * write_data, uint16_t data_length)
{
    uint8_t TxAddr[2] = {0,0};

    while(!I2C_ALLOW);		//wait until it is safe to transmit

    TxAddr[0] = reg_addr >> 8; 		//MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	//LSB of address

    RF430_I2C_Write((uint8_t *)TxAddr, 2, START);

    RF430_I2C_Write((uint8_t  *)write_data, data_length, CONTINUE);

    RF430_I2C_Send_Stop();

    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent
}

/**************************************************************************************************************************************************
*   RF430_I2C_Write
***************************************************************************************************************************************************
*
* Brief : Writes data over I2C
*
* Param[in] :   data: the data array to write
* 				length: the amount of data to write
* 				cont: if set to START will issue a Start condition on the I2C bus.  Other option is CONTINUE (data write without a Start)
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void RF430_I2C_Write(uint8_t  * data, uint16_t length, uint16_t cont)
{
	__delay_cycles(I2C_DELAY);                   // Delay required between transaction

    PTxData = (uint8_t  *)data;      // TX array start address

    TXByteCtr = length;

    if (cont == START) {
        RF430_I2C_Start = SET;					// need a way of knowing when a start condition has been set in the ISR AK 10-23-2013
        UCB0CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    }
    else {
        UCB0CTL1 |= UCTR;                       // I2C TX
        UCB0IFG |= UCTXIFG;                     // Kickoff IFG since START not sent
    }

    __bis_SR_register(LPM_MODE + GIE);     // Enter LPM, enable interrupts
    __no_operation();                       // Remain in LPM until all data
                                            // is TX'd
    RF430_I2C_Start = CLEAR;				// this should be cleared in the interrupt, but to be sure it is also cleared here, AK-1023-2013

    if (!(RF430_I2C_State == I2C_NACK_RCVD))
    {
        while (TXByteCtr != 0);          // Ensure stop condition got sent
    }

}

/**************************************************************************************************************************************************
*   RF430_I2C_Read
***************************************************************************************************************************************************
*
* Brief : Reads out data over I2C
*
* Param[in] :   data: the array to store the read out data
* 				length: the amount of data to read
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void RF430_I2C_Read(uint8_t* data, uint16_t length)
{
    __delay_cycles(I2C_DELAY);                   // Delay required between transaction

    PRxData = (uint8_t *)data;    // Start of RX buffer
    RXByteCtr = length;                          // Load RX byte counter

    UCB0CTL1 &= ~UCTR;                      // I2C Read
    UCB0CTL1 |= UCTXSTT;                    // I2C start condition

    __bis_SR_register(LPM_MODE + GIE);     // Enter LPM, enable interrupts
                                            // Remain in LPM until all data
                                            // is RX'd
    __no_operation();                       // Set breakpoint >>here<< and
                                            // read out the RxBuffer buffer
}

/**************************************************************************************************************************************************
*   RF430_I2C_Write_Restart_Read
***************************************************************************************************************************************************
*
* Brief : First sends out a portion of data (tx_data) and then recieves the data transmitted by the RF430CL331H
*
* Param[in] :   tx_data: the array to data to transmit
* 				tx_length: the amount of data to transmit
* 				rx_data: the array of data to recieve (after the transmit)
* 				rx_length: the amount of data to receive
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
uint8_t RF430_I2C_Write_Restart_Read(uint8_t  * tx_data, uint16_t tx_length, uint8_t* rx_data, uint16_t rx_length)
{
    RF430_I2C_State = I2C_TRANSMIT;

    PTxData = (uint8_t  *)tx_data;     // TX array start address
    TXByteCtr = tx_length;

    RF430_I2C_Start = SET;					// need a way of knowing when a start condition has been set in the ISR AK 10-23-2013
    UCB0CTL1 |= UCTR + UCTXSTT;             // I2C Master Transmitter Mode, Generate START Condition

    if(RF430_I2C_State == I2C_NACK_RCVD)    // If RF430 NACKed
    {
        //Debug_Print("RF430 NACKed!!!!!!!\n");
        return FAIL;
    }

    __bis_SR_register(LPM_MODE + GIE);     // Enter LPM, enable interrupts
    __no_operation();                       // Remain in LPM until all data

    RF430_I2C_Start = CLEAR;				// this should be cleared in the interrupt, but to be sure it is also cleared here, AK-1023-2013

    if(RF430_I2C_State == I2C_NACK_RCVD)    // If RF430 NACKed
    {
        //Debug_Print("RF430 NACKed!!!!!!!\n");
        return FAIL;
    }

    PRxData = (uint8_t *)rx_data;           // Start of RX buffer
    RXByteCtr = rx_length;                          // Load RX byte counter

    UCB0CTL1 &= ~UCTR;                      // I2C Read
    RF430_I2C_Start = CLEAR;				// only needed on transmit and start condition --AK 10-23-2013
    UCB0CTL1 |= UCTXSTT;                    // I2C start condition

    __bis_SR_register(LPM_MODE + GIE);     // Enter LPM, enable interrupts
                                            // Remain in LPM until all data
                                            // is RX'd
    RF430_I2C_Start = CLEAR;				// this should be cleared in the interrupt, but to be sure it is also cleared here, AK-1023-2013
    __no_operation();                       // Set breakpoint >>here<< and
                                            // read out the RxBuffer buffer
    return PASS;
}

/**************************************************************************************************************************************************
*   RF430_I2C_Send_Stop
***************************************************************************************************************************************************
*
* Brief : Sends out a Stop condition to terminate a packet
*
* Param[in] :   None
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void RF430_I2C_Send_Stop()
{
    __delay_cycles(100);
    UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
    __enable_interrupt();
    while (!(UCB0CTL1 & UCTXSTP));             // Ensure stop condition got sent
    __disable_interrupt();
}


/**************************************************************************************************************************************************
*   USCI_B0_ISR
***************************************************************************************************************************************************
*
* Brief : Handles sending and recieving packets of data over I2C.  Controlled by the above functions.
*
* Param[in] :   None
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{
  switch(__even_in_range(UCB0IV,12))
  {
  case  USCI_NONE:
	  break;                           		// No interrupts
  case  USCI_I2C_UCALIFG:					// UCALIFG interrupt
	  break;
  case  USCI_I2C_UCNACKIFG:					// NACK recieved
    RF430_I2C_State = I2C_NACK_RCVD;
    UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
    UCB0IFG = 0;                            // Clear All USCI_B0 flags
    __bic_SR_register_on_exit(LPM_MODE);    // Exit LPM
    break;
  case  USCI_I2C_UCSTTIFG:					// Start condition
	  break;
  case  USCI_I2C_UCSTPIFG:					// Stop condition
	  break;
  case USCI_I2C_UCRXIFG:                 	// Handles RX data
    RXByteCtr--;                            // Decrement RX byte counter
    if (RXByteCtr)
    {
      *PRxData++ = UCB0RXBUF;               // Move RX data to address PRxData
      if (RXByteCtr == 1)
        UCB0CTL1 |= UCTXSTP;                // Generate I2C stop condition on next RX
    }
    else
    {
      *PRxData = UCB0RXBUF;                 // Move final RX data to PRxData
      __bic_SR_register_on_exit(LPM_MODE); // Exit active CPU
    }
    break;
  case USCI_I2C_UCTXIFG:                    // Handles TX data
    if (TXByteCtr)                          // Check TX byte counter
    {
      unsigned int delay = 0;
      UCB0TXBUF = *PTxData++;               // Load TX buffer
      TXByteCtr--;                          // Decrement TX byte counter
      if(TXByteCtr == 1)
      {
    	  while((delay < 9000) && (UCB0CTL1 & UCTXSTT) && (RF430_I2C_Start == SET)) //only go into this loop when a start condition has been requested
    	  {
    		  __delay_cycles(40);
    		  delay += 40;
    	  }
		  if ((UCB0CTL1 & UCTXSTT) && (RF430_I2C_Start == SET))            // Still Waiting?  --only when start condition is expected,
		  {
			  RF430_I2C_State = I2C_NACK_RCVD;
			  __bic_SR_register_on_exit(LPM_MODE); // Exit LPM
		  }
      }
    }
    else
    {
      //UCB0CTL1 |= UCTXSTP;                  // I2C stop condition
      UCB0IFG &= ~UCTXIFG;                  // Clear USCI_B0 TX int flag
      __bic_SR_register_on_exit(LPM_MODE); // Exit LPM
    }
    RF430_I2C_Start = CLEAR;			//Clear after the first TX, start has been passed
    break;
  default:
    __no_operation();
    break;
  }
}
