/*
 * Request_Processor.c
 *
 * RF430CL331H Host Example Project
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/


#include "Request_Processor.h"
#include "RF430_example.h"
#include "RF430_Comm.h"
#include <stdint.h>
#include "msp430.h"

struct NdefFile_Type NdefFiles[3];
uint16_t NumberOfFiles = 0;
const uint16_t E104_Length;

uint8_t CCFileText[100] = 	{
							0x00, 0x0F,	/* CCLEN */
							0x20,		/* Mapping version 2.0 */
							0x00, 0xF9,	/* MLe (249 bytes); Maximum R-APDU data size */
							0x00, 0xF6, /* MLc (246 bytes); Maximum C-APDU data size */
							0x04, 		/* Tag, File Control TLV (4 = NDEF file) */
							0x06, 		/* Length, File Control TLV (6 = 6 bytes of data for this tag) */
							0xE1, 0x04,	/* File Identifier */
							0x23, 0x1E, /* Max NDEF size (8990 bytes of useable memory) */
							0x00, 		/* NDEF file read access condition, read access without any security */
							0x00 		/* NDEF file write access condition; write access without any security */
							};//CC file text

uint8_t FileTextE104[9000] = {
							0x00, 0x16, /* NLEN; NDEF length (3 byte long message) */
							0xD1, 0x01, 0x12,
							0x54, /* T = text */
							0x02,
							0x65, 0x6E, /* 'e', 'n', */

							/* 'Hello, world!' NDEF data; */
							0x48, 0x65, 0x6C, 0x6C, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21

};//Ndef file text

uint16_t SelectedFile;		//the file that is currently selected

/**************************************************************************************************************************************************
*   AppInit
***************************************************************************************************************************************************
*
* Brief : Initializes the file management
*
* Param[in] :   None
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void AppInit()
{
	// Init CC file info
	NdefFiles[0].FileID[0] = 0xE1;
	NdefFiles[0].FileID[1] = 0x03;
	NdefFiles[0].FilePointer = (uint8_t *)CCFileText;
	NdefFiles[0].FileLength = 0;		//?


	// Init Ndef file info
	NdefFiles[1].FileID[0] = 0xE1;
	NdefFiles[1].FileID[1] = 0x04;
	NdefFiles[1].FilePointer = (uint8_t *)FileTextE104;
	//NdefFiles[1].FileLength = 0;		//?

	NumberOfFiles = 2; 			//the number if NDEF files available
	SelectedFile = 0;			//default to CC file
}


/**************************************************************************************************************************************************
*   AppInit
***************************************************************************************************************************************************
*
* Brief : Service an interrupt from the RF430
*
* Param[in] :   flags: the interrupt flags read out from the RF430CL331H device
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void ServiceInterrupt(uint16_t flags)
{
	uint16_t interrupt_serviced = 0;

	if(flags & GENERIC_ERROR_INT_FLAG) 									// check if the there was a reset on the RF430
	{
		interrupt_serviced |= GENERIC_ERROR_INT_FLAG;
		Write_Register(INT_FLAG_REG, interrupt_serviced); 				// clear this flag
	}

	if(flags & FIELD_REMOVED_INT_FLAG) 									// check if the tag was removed
	{
		interrupt_serviced |= FIELD_REMOVED_INT_FLAG;					// clear this flag later
		Write_Register(INT_FLAG_REG, interrupt_serviced); //ACK the flags to clear
	}

	if(flags & DATA_TRANSACTION_INT_FLAG) //check if the tag was read
	{
		uint16_t status;
		uint16_t ret;
		status = Read_Register(STATUS_REG); //read the status register to determine the nature of the interrupt

		switch (status & APP_STATUS_REGS)
		{
			// NDEF File Select Request is coming from the mobile/reader - response to the request is determined here
			// based on whether the file exists in our file database
			case FILE_SELECT_STATUS:
			{
				uint16_t file_id;
				file_id = Read_Register(NDEF_FILE_ID);						// determine the file identifier of the select request
				ret = SearchForFile((uint8_t *)&file_id);				// determine if the file exists on the host MCU
				interrupt_serviced |= DATA_TRANSACTION_INT_FLAG;			// clear this flag later
				if (ret == FileFound)
				{
					Write_Register(INT_FLAG_REG, interrupt_serviced); 		// ACK the flags to clear
					Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD + FILE_EXISTS_FIELD);			// indicate to the RF430 that the file exist
				}
				else
				{
					Write_Register(INT_FLAG_REG, interrupt_serviced); 		// ACK the flags to clear
					Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD + FILE_DOES_NOT_EXIST_FIELD);	// the file does not exist
				}
				break;
			}

			// NDEF ReadBinary request has been sent by the mobile / reader
			case FILE_REQUEST_STATUS:
			{
				uint16_t buffer_start;
				uint16_t file_offset;
				uint16_t file_length;
				buffer_start = Read_Register(NDEF_BUFFER_START);		// where to start writing the file info in the RF430 buffer (0-2999)
				file_offset = Read_Register(NDEF_FILE_OFFSET);			// what part of the file to start sending
				file_length = Read_Register(NDEF_FILE_LENGTH);			// how much of the file starting at offset to send
																		// we can send more than requested, called caching
																		// as long as we write back into the length register how
																		// much we sent it
				interrupt_serviced |= DATA_TRANSACTION_INT_FLAG;		// clear this flag later
				// can have bounds check for the requested length
				file_length = SendDataOnFile(SelectedFile, buffer_start, file_offset, file_length);

				Write_Register(NDEF_FILE_LENGTH, file_length);  		// how much was actually written
				Write_Register(INT_FLAG_REG, interrupt_serviced); 		// ACK the flags to clear
				Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD);		// indicate that we have serviced the request

				// custom response code
				// Write_Register(CUSTOM_RESPONSE_REG, 0x6491);  		// error code
				// Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD + CUSTOM_RESPONSE_FIELD);
				break;
			}

			// NDEF UpdateBinary request
			case FILE_AVAILABLE_STATUS:
			{
				uint16_t buffer_start;
				uint16_t file_offset;
				uint16_t file_length;
				interrupt_serviced |= DATA_TRANSACTION_INT_FLAG;			// clear this flag later
				buffer_start = Read_Register(NDEF_BUFFER_START);			// where to start in the RF430 buffer to read the file data (0-2999)
				file_offset = Read_Register(NDEF_FILE_OFFSET);				// the file offset that the data begins at
				file_length = Read_Register(NDEF_FILE_LENGTH);				// how much of the file is in the RF430 buffer

				//can have bounds check for the requested length
				ReadDataOnFile(SelectedFile, buffer_start, file_offset, file_length);
				Write_Register(INT_FLAG_REG, interrupt_serviced); 			// ACK the flags to clear
				Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD);			// the interrup has been serviced

				// custom response code
				// Write_Register(CUSTOM_RESPONSE_REG, 0x6491);  	//error code
				// Write_Register(HOST_RESPONSE, INT_SERVICED_FIELD + CUSTOM_RESPONSE_FIELD);
				break;
			}
		}
	}

	// This is the pre-fetch interrupt
	// If this is enabled with the interrupt enable, the RF430 will request more read data when it is sending the previous
	// ReadBinary request.  The goal is to have enough data in the RF430 buffer for the next ReadBinary request so that
	// it responds the ReadBinary request automatically, with no interrupt.
	else if(flags & EXTRA_DATA_IN_FLAG) //check if the tag was read
	{
		#define AMOUNT_DATA_TO_SENT_EARLY		255							// data to add to the buffer while transmit is happening
		#define MAX_TAG_BUFFER_SPACE			2998						// actually 3000 but to avoid any possibility of an issue

		uint16_t buffer_start;
		uint16_t file_offset;
		uint16_t file_length = 0;

		interrupt_serviced |= EXTRA_DATA_IN_FLAG;							// clear the interrupt flag

		if (SelectedFile == 1)   											// allows only prefetch on NDEF files, no Capability container ones
		{
			buffer_start = Read_Register(NDEF_BUFFER_START);
			file_offset = Read_Register(NDEF_FILE_OFFSET);
			//		file_length = Read_Register(NDEF_FILE_LENGTH);

			if ((buffer_start + AMOUNT_DATA_TO_SENT_EARLY) >= MAX_TAG_BUFFER_SPACE)
			{
				// can't fill the buffer anymore
				// do no fill.  New data request, the remaining data will be shifted to the beggining of the buffer, interrupt will come later.
				__no_operation();
			}
			else
			{
				file_length = SendDataOnFile(SelectedFile, buffer_start, file_offset, AMOUNT_DATA_TO_SENT_EARLY); //255 is enough for atleast one packet
			}
		}
		Write_Register(NDEF_FILE_LENGTH, file_length);  						// how much was actually written
		Write_Register(INT_FLAG_REG, interrupt_serviced);						// ACK the flags to clear
		Write_Register(HOST_RESPONSE, EXTRA_DATA_IN_SENT_FIELD);				// interrupt was serviced
	}
}

/**************************************************************************************************************************************************
*   SearchForFile
***************************************************************************************************************************************************
*
* Brief : Searches the internal file array for a request file.
*
* Param[in] :   fileId: the interrupt flags read out from the RF430CL331H device
*
* Param[out]:   None
*
* Return :      FileExistType: FileFound is returned when a file is found, otherwise FileNotFound is returned
**************************************************************************************************************************************************/
enum FileExistType SearchForFile(uint8_t *fileId)
{
	uint16_t i;
	enum FileExistType ret = FileNotFound; // 0 means not found, 1 mean found

	for (i = 0; i < NumberOfFiles; i++)
	{
		if (NdefFiles[i].FileID[0] == fileId[0] && NdefFiles[i].FileID[1] == fileId[1])
		{
			ret = FileFound;
			SelectedFile = i;  // the index of the selected file in the array
			break;
		}
	}
	return ret;
}

/**************************************************************************************************************************************************
*   SendDataOnFile
***************************************************************************************************************************************************
*
* Brief : Tag is requesting data on the selected file.  This would be a result of the ReadBinary NDEF command. Return the data requested.
*
* Param[in] :   selectedFile: the selected file ID.
* 				buffer_start: where in the RF430CL331H SRAM location to start placing the data
* 				file_offset: The location in the file where to start sending the requested data
* 				length: The amount of data to send from the requested file
*
* Param[out]:   None
*
* Return :      ret_length: returns how much actual data was sent out
**************************************************************************************************************************************************/
uint16_t SendDataOnFile(uint16_t selectedFile, uint16_t buffer_start, uint16_t file_offset, uint16_t length)
{
	uint16_t ret_length;
	uint16_t file_length;

	file_length = (((uint16_t)NdefFiles[selectedFile].FilePointer[0]) << 8) + NdefFiles[selectedFile].FilePointer[1];  // reads the NLEN of the file

	// make sure we are not sending above the length of the file (the send data early feature above does not do file length check)
	// if reader requests data we can expect that it is the correct length
	if (file_length < (file_offset + length))
	{
		length = file_length - file_offset;
	}

	Write_Continuous (buffer_start, (uint8_t *)&NdefFiles[selectedFile].FilePointer[file_offset], length);
	ret_length = length;

	return ret_length;

	//Regs[NDEF_FILE_LENGTH_INDEX]  = Regs[NDEF_FILE_LENGTH_INDEX]
	//do not change right now, we are only sending as much as has been requested
	//if we wanted to send data than requested, we would update the Regs[NDEF_FILE_LENGTH_INDEX] register to a higher value
}

/**************************************************************************************************************************************************
*   ReadDataOnFile
***************************************************************************************************************************************************
*
* Brief : This would be a result of the UpdateBinary NDEF command. Tag has data to be read out on the selected file. This function reads out that data
*
* Param[in] :   selectedFile: the selected file ID.
* 				buffer_start: where in the RF430CL331H SRAM location to start placing the data
* 				file_offset: The location in the file where to start sending the requested data
* 				length: The amount of data to send from the requested file
*
* Param[out]:   None
*
* Return :      None
**************************************************************************************************************************************************/
void ReadDataOnFile(uint16_t selectedFile, uint16_t buffer_start, uint16_t file_offset, uint16_t length)
{
	uint16_t * e104_l = (uint16_t *)&E104_Length;

	Read_Continuous(buffer_start, (uint8_t *)&NdefFiles[selectedFile].FilePointer[file_offset], length);
	if (NdefFiles[selectedFile].FileLength < (file_offset + length))
	{
		NdefFiles[selectedFile].FileLength = file_offset + length;
		*e104_l = file_offset + length;
	}
	//Regs[NDEF_FILE_LENGTH_INDEX]  = Regs[NDEF_FILE_LENGTH_INDEX]
	//do not change right now, we are only sending as much as has been requested
	//if we wanted to send more data than requested, we would update the Regs[NDEF_FILE_LENGTH_INDEX] register to a higher value
}
